@@ -1,7 +1,4 @@
-try:
-    from hy.version import __version__
-except ImportError:
-    __version__ = "unknown"
+__version__ = '0.29.0'
 
 
 def _initialize_env_var(env_var, default_val):
@@ -33,6 +30,7 @@ I = I()
 # to be loaded if they're not needed.
 
 _jit_imports = dict(
+    pyops=["hy.pyops", None],
     read="hy.reader",
     read_many="hy.reader",
     mangle="hy.reader",
@@ -46,22 +44,20 @@ _jit_imports = dict(
     disassemble="hy.core.util",
     as_model="hy.models",
     REPL="hy.repl",
+    Reader="hy.reader.reader",
+    HyReader="hy.reader.hy_reader",
+    PrematureEndOfInput="hy.reader.exceptions"
 )
 
 
 def __getattr__(k):
-    if k == "pyops":
-        global pyops
-        import hy.pyops
-
-        pyops = hy.pyops
-        return pyops
-
     if k not in _jit_imports:
         raise AttributeError(f"module {__name__!r} has no attribute {k!r}")
+
     v = _jit_imports[k]
     module, original_name = v if isinstance(v, list) else (v, k)
     import importlib
+    module = importlib.import_module(module)
 
-    globals()[k] = getattr(importlib.import_module(module), original_name)
+    globals()[k] = getattr(module, original_name) if original_name else module
     return globals()[k]
