@@ -4,11 +4,11 @@ from textwrap import dedent
 import pytest
 
 import hy
-from hy._compat import PY3_11
+from hy.compat import PY3_11
 from hy.compiler import hy_compile
 from hy.errors import HyError, HyLanguageError
 from hy.reader import read_many
-from hy.reader.exceptions import LexException, PrematureEndOfInput
+from hy.reader.exceptions import LexException
 
 
 def _ast_spotcheck(arg, root, secondary):
@@ -466,7 +466,7 @@ def test_compile_error():
 
 def test_for_compile_error():
     """Ensure we get compile error in tricky 'for' cases"""
-    with pytest.raises(PrematureEndOfInput) as excinfo:
+    with pytest.raises(hy.PrematureEndOfInput) as excinfo:
         can_compile("(fn [] (for)")
     assert excinfo.value.msg.startswith("Premature end of input")
 
@@ -550,11 +550,12 @@ def test_compiler_macro_tag_try():
 
 
 def test_ast_good_yield_from():
-    can_compile("(yield-from [1 2])")
+    can_compile("(yield :from [1 2])")
+    can_compile("(yield :from)")
 
 
 def test_ast_bad_yield_from():
-    cant_compile("(yield-from)")
+    cant_compile("(yield :ploopy [1 2])")
 
 
 def test_eval_generator_with_return():
