@@ -11,7 +11,7 @@ from pathlib import Path
 
 import pytest
 
-from hy._compat import PY3_9, PYODIDE
+from hy.compat import PY3_9, PYODIDE
 
 if PYODIDE:
     pytest.skip(
@@ -25,8 +25,8 @@ def pyr(s=""):
 
 def run_cmd(
         cmd, stdin_data=None, expect=0, dontwritebytecode=False,
-        cwd=None, stdout=subprocess.PIPE):
-    env = dict(os.environ)
+        cwd=None, stdout=subprocess.PIPE, env=None):
+    env = {**dict(os.environ), **(env or {})}
     if dontwritebytecode:
         env["PYTHONDONTWRITEBYTECODE"] = "1"
     else:
@@ -80,6 +80,18 @@ def test_stdin():
     assert "TU" in out
 
 
+def test_i_flag_repl_env():
+    # If a program is passed in through standard input, it's evaluated
+    # in the REPL environment.
+    code = '(import sys) (if (hasattr sys "ps1") "Yeppers" "Nopers")'
+    out, _ = run_cmd("hy -i", code)
+    assert "Yeppers" in out
+    # With `-c`, on the other hand, the code is run before the REPL is
+    # launched.
+    out, _ = run_cmd(['hy', '-i', '-c', code])
+    assert "Nopers" in out
+
+
 def test_error_parts_length():
     """Confirm that exception messages print arrows surrounding the affected
     expression."""
@@ -149,21 +161,20 @@ def test_mangle_m():
 
 
 def test_ignore_python_env():
-    os.environ.update({"PYTHONTEST": "0"})
-    output, _ = run_cmd("hy -c '(print (do (import os) (. os environ)))'")
+    e = dict(PYTHONTEST = "0")
+
+    output, _ = run_cmd("hy -c '(print (do (import os) (. os environ)))'", env = e)
     assert "PYTHONTEST" in output
-    output, _ = run_cmd("hy -m tests.resources.bin.printenv")
+    output, _ = run_cmd("hy -m tests.resources.bin.printenv", env = e)
     assert "PYTHONTEST" in output
-    output, _ = run_cmd("hy tests/resources/bin/printenv.hy")
+    output, _ = run_cmd("hy tests/resources/bin/printenv.hy", env = e)
     assert "PYTHONTEST" in output
 
-    output, _ = run_cmd("hy -E -c '(print (do (import os) (. os environ)))'")
+    output, _ = run_cmd("hy -E -c '(print (do (import os) (. os environ)))'", env = e)
     assert "PYTHONTEST" not in output
-    os.environ.update({"PYTHONTEST": "0"})
-    output, _ = run_cmd("hy -E -m tests.resources.bin.printenv")
+    output, _ = run_cmd("hy -E -m tests.resources.bin.printenv", env = e)
     assert "PYTHONTEST" not in output
-    os.environ.update({"PYTHONTEST": "0"})
-    output, _ = run_cmd("hy -E tests/resources/bin/printenv.hy")
+    output, _ = run_cmd("hy -E tests/resources/bin/printenv.hy", env = e)
     assert "PYTHONTEST" not in output
 
 
@@ -447,7 +458,7 @@ def test_tracebacks():
         r'  File "(?:<string>|string-[0-9a-f]+)", line 1\n'
         r'    \(print "\n'
         r"           \^\n"
-        r"hy.reader.exceptions.PrematureEndOfInput"
+        r"hy.PrematureEndOfInput"
     )
     assert re.search(peoi_re, error)
 
@@ -498,34 +509,34 @@ def test_traceback_shebang(tmp_path):
 
 def test_hystartup():
     # spy == True and custom repl-output-fn
-    os.environ["HYSTARTUP"] = "tests/resources/hystartup.hy"
-    output, _ = run_cmd("hy -i", "[1 2]")
+    env = dict(HYSTARTUP = "tests/resources/hystartup.hy")
+    output, _ = run_cmd("hy -i", "[1 2]", env = env)
+    assert "p1? " in output
     assert "[1, 2]" in output
     assert "[1,_2]" in output
 
-    output, _ = run_cmd("hy -i", "(hello-world)")
+    output, _ = run_cmd("hy -i", "(hello-world)", env = env)
     assert "(hello-world)" not in output
     assert "1 + 1" in output
     assert "2" in output
 
-    output, _ = run_cmd("hy -i", "#rad")
+    output, _ = run_cmd("hy -i", "#rad", env = env)
     assert "#rad" not in output
     assert "'totally' + 'rad'" in output
     assert "'totallyrad'" in output
 
-    output, _ = run_cmd("hy -i --repl-output-fn repr", "[1 2 3 4]")
+    output, _ = run_cmd("hy -i --repl-output-fn repr", "[1 2 3 4]", env = env)
     assert "[1, 2, 3, 4]" in output
     assert "[1 2 3 4]" not in output
     assert "[1,_2,_3,_4]" not in output
 
     # spy == False and custom repl-output-fn
-    os.environ["HYSTARTUP"] = "tests/resources/spy_off_startup.hy"
-    output, _ = run_cmd("hy -i --spy", "[1 2]")  # overwrite spy with cmdline arg
+    # Then overwrite spy with cmdline arg
+    output, _ = run_cmd("hy -i --spy", "[1 2]",
+         env = dict(HYSTARTUP = "tests/resources/spy_off_startup.hy"))
     assert "[1, 2]" in output
     assert "[1,~2]" in output
 
-    del os.environ["HYSTARTUP"]
-
 
 def test_output_buffering(tmp_path):
     tf = tmp_path / "file.txt"
